@@ -9,6 +9,7 @@ import nest_asyncio
 from promptlayer.groups import AsyncGroupManager, GroupManager
 from promptlayer.promptlayer_base import PromptLayerBase
 from promptlayer.promptlayer_mixins import PromptLayerMixin
+from promptlayer.streaming import astream_response, stream_response
 from promptlayer.templates import AsyncTemplateManager, TemplateManager
 from promptlayer.track import AsyncTrackManager, TrackManager
 from promptlayer.types.prompt_template import PromptTemplate
@@ -16,10 +17,8 @@ from promptlayer.utils import (
     RERAISE_ORIGINAL_EXCEPTION,
     _get_workflow_workflow_id_or_name,
     arun_workflow_request,
-    astream_response,
     atrack_request,
     autil_log_request,
-    stream_response,
     track_request,
     util_log_request,
 )
@@ -136,6 +135,7 @@ class PromptLayer(PromptLayerMixin):
             metadata=metadata,
             provider=provider,
             model=model,
+            model_parameter_overrides=model_parameter_overrides,
         )
         prompt_blueprint = self.templates.get(prompt_name, get_prompt_template_params)
         prompt_blueprint_model = self._validate_and_extract_model_from_prompt_blueprint(
@@ -145,7 +145,6 @@ class PromptLayer(PromptLayerMixin):
             prompt_blueprint=prompt_blueprint,
             prompt_template=prompt_blueprint["prompt_template"],
             prompt_blueprint_model=prompt_blueprint_model,
-            model_parameter_overrides=model_parameter_overrides,
             stream=stream,
         )
 
@@ -169,8 +168,14 @@ class PromptLayer(PromptLayerMixin):
                     pl_run_span_id=pl_run_span_id,
                 ),
                 map_results=llm_data["stream_function"],
+                metadata=llm_data["prompt_blueprint"]["metadata"],
             )
 
+        if isinstance(response, dict):
+            request_response = response
+        else:
+            request_response = response.model_dump(mode="json")
+
         request_log = self._track_request_log(
             llm_data,
             tags,
@@ -178,7 +183,7 @@ class PromptLayer(PromptLayerMixin):
             group_id,
             pl_run_span_id,
             metadata=metadata,
-            request_response=response.model_dump(mode="json"),
+            request_response=request_response,
         )
 
         return {
@@ -320,6 +325,8 @@ class PromptLayer(PromptLayerMixin):
         price: float = 0.0,
         function_name: str = "",
         score: int = 0,
+        prompt_id: Union[int, None] = None,
+        score_name: Union[str, None] = None,
     ):
         return util_log_request(
             self.api_key,
@@ -340,6 +347,8 @@ class PromptLayer(PromptLayerMixin):
             price=price,
             function_name=function_name,
             score=score,
+            prompt_id=prompt_id,
+            score_name=score_name,
         )
 
 
@@ -478,6 +487,7 @@ class AsyncPromptLayer(PromptLayerMixin):
         price: float = 0.0,
         function_name: str = "",
         score: int = 0,
+        prompt_id: Union[int, None] = None,
     ):
         return await autil_log_request(
             self.api_key,
@@ -498,6 +508,7 @@ class AsyncPromptLayer(PromptLayerMixin):
             price=price,
             function_name=function_name,
             score=score,
+            prompt_id=prompt_id,
         )
 
     async def _create_track_request_callable(
@@ -568,6 +579,7 @@ class AsyncPromptLayer(PromptLayerMixin):
             metadata=metadata,
             provider=provider,
             model=model,
+            model_parameter_overrides=model_parameter_overrides,
         )
         prompt_blueprint = await self.templates.get(prompt_name, get_prompt_template_params)
         prompt_blueprint_model = self._validate_and_extract_model_from_prompt_blueprint(
@@ -577,7 +589,6 @@ class AsyncPromptLayer(PromptLayerMixin):
             prompt_blueprint=prompt_blueprint,
             prompt_template=prompt_blueprint["prompt_template"],
             prompt_blueprint_model=prompt_blueprint_model,
-            model_parameter_overrides=model_parameter_overrides,
             stream=stream,
             is_async=True,
         )
@@ -588,6 +599,11 @@ class AsyncPromptLayer(PromptLayerMixin):
             function_kwargs=llm_data["function_kwargs"],
         )
 
+        if hasattr(response, "model_dump"):
+            request_response = response.model_dump(mode="json")
+        else:
+            request_response = response
+
         if stream:
             track_request_callable = await self._create_track_request_callable(
                 request_params=llm_data,
@@ -597,9 +613,10 @@ class AsyncPromptLayer(PromptLayerMixin):
                 pl_run_span_id=pl_run_span_id,
             )
             return astream_response(
-                response,
+                request_response,
                 track_request_callable,
                 llm_data["stream_function"],
+                llm_data["prompt_blueprint"]["metadata"],
             )
 
         request_log = await self._track_request_log(
@@ -609,11 +626,11 @@ class AsyncPromptLayer(PromptLayerMixin):
             group_id,
             pl_run_span_id,
             metadata=metadata,
-            request_response=response.model_dump(mode="json"),
+            request_response=request_response,
         )
 
         return {
             "request_id": request_log.get("request_id", None),
-            "raw_response": response,
+            "raw_response": request_response,
             "prompt_blueprint": request_log.get("prompt_blueprint", None),
         }
