@@ -10,37 +10,47 @@ from opentelemetry.sdk.trace.export import BatchSpanProcessor
 from opentelemetry.semconv.resource import ResourceAttributes
 
 from promptlayer.span_exporter import PromptLayerSpanExporter
-from promptlayer.utils import (
-    aanthropic_request,
+from promptlayer.streaming import (
     aanthropic_stream_completion,
     aanthropic_stream_message,
-    aazure_openai_request,
-    agoogle_request,
+    abedrock_stream_message,
     agoogle_stream_chat,
     agoogle_stream_completion,
-    amistral_request,
     amistral_stream_chat,
-    anthropic_request,
     anthropic_stream_completion,
     anthropic_stream_message,
-    aopenai_request,
+    aopenai_responses_stream_chat,
     aopenai_stream_chat,
     aopenai_stream_completion,
+    bedrock_stream_message,
+    google_stream_chat,
+    google_stream_completion,
+    mistral_stream_chat,
+    openai_responses_stream_chat,
+    openai_stream_chat,
+    openai_stream_completion,
+)
+from promptlayer.utils import (
+    aamazon_bedrock_request,
+    aanthropic_bedrock_request,
+    aanthropic_request,
+    aazure_openai_request,
+    agoogle_request,
+    amazon_bedrock_request,
+    amistral_request,
+    anthropic_bedrock_request,
+    anthropic_request,
+    aopenai_request,
     avertexai_request,
     azure_openai_request,
     google_request,
-    google_stream_chat,
-    google_stream_completion,
     mistral_request,
-    mistral_stream_chat,
     openai_request,
-    openai_stream_chat,
-    openai_stream_completion,
     vertexai_request,
 )
 
 MAP_PROVIDER_TO_FUNCTION_NAME = {
-    "openai": {
+    "openai:chat-completions": {
         "chat": {
             "function_name": "openai.chat.completions.create",
             "stream_function": openai_stream_chat,
@@ -50,6 +60,16 @@ MAP_PROVIDER_TO_FUNCTION_NAME = {
             "stream_function": openai_stream_completion,
         },
     },
+    "openai:responses": {
+        "chat": {
+            "function_name": "openai.responses.create",
+            "stream_function": openai_responses_stream_chat,
+        },
+        "completion": {
+            "function_name": "openai.responses.create",
+            "stream_function": openai_responses_stream_chat,
+        },
+    },
     "anthropic": {
         "chat": {
             "function_name": "anthropic.messages.create",
@@ -60,7 +80,7 @@ MAP_PROVIDER_TO_FUNCTION_NAME = {
             "stream_function": anthropic_stream_completion,
         },
     },
-    "openai.azure": {
+    "openai.azure:chat-completions": {
         "chat": {
             "function_name": "openai.AzureOpenAI.chat.completions.create",
             "stream_function": openai_stream_chat,
@@ -70,6 +90,16 @@ MAP_PROVIDER_TO_FUNCTION_NAME = {
             "stream_function": openai_stream_completion,
         },
     },
+    "openai.azure:responses": {
+        "chat": {
+            "function_name": "openai.AzureOpenAI.responses.create",
+            "stream_function": openai_responses_stream_chat,
+        },
+        "completion": {
+            "function_name": "openai.AzureOpenAI.responses.create",
+            "stream_function": openai_responses_stream_chat,
+        },
+    },
     "mistral": {
         "chat": {
             "function_name": "mistral.client.chat",
@@ -90,6 +120,26 @@ MAP_PROVIDER_TO_FUNCTION_NAME = {
             "stream_function": google_stream_completion,
         },
     },
+    "amazon.bedrock": {
+        "chat": {
+            "function_name": "boto3.bedrock-runtime.converse",
+            "stream_function": bedrock_stream_message,
+        },
+        "completion": {
+            "function_name": "boto3.bedrock-runtime.converse",
+            "stream_function": bedrock_stream_message,
+        },
+    },
+    "anthropic.bedrock": {
+        "chat": {
+            "function_name": "anthropic.messages.create",
+            "stream_function": anthropic_stream_message,
+        },
+        "completion": {
+            "function_name": "anthropic.completions.create",
+            "stream_function": anthropic_stream_completion,
+        },
+    },
 }
 
 
@@ -100,10 +150,12 @@ MAP_PROVIDER_TO_FUNCTION = {
     "openai": openai_request,
     "openai.azure": azure_openai_request,
     "vertexai": vertexai_request,
+    "amazon.bedrock": amazon_bedrock_request,
+    "anthropic.bedrock": anthropic_bedrock_request,
 }
 
 AMAP_PROVIDER_TO_FUNCTION_NAME = {
-    "openai": {
+    "openai:chat-completions": {
         "chat": {
             "function_name": "openai.chat.completions.create",
             "stream_function": aopenai_stream_chat,
@@ -113,6 +165,16 @@ AMAP_PROVIDER_TO_FUNCTION_NAME = {
             "stream_function": aopenai_stream_completion,
         },
     },
+    "openai:responses": {
+        "chat": {
+            "function_name": "openai.responses.create",
+            "stream_function": aopenai_responses_stream_chat,
+        },
+        "completion": {
+            "function_name": "openai.responses.create",
+            "stream_function": aopenai_responses_stream_chat,
+        },
+    },
     "anthropic": {
         "chat": {
             "function_name": "anthropic.messages.create",
@@ -123,7 +185,7 @@ AMAP_PROVIDER_TO_FUNCTION_NAME = {
             "stream_function": aanthropic_stream_completion,
         },
     },
-    "openai.azure": {
+    "openai.azure:chat-completions": {
         "chat": {
             "function_name": "openai.AzureOpenAI.chat.completions.create",
             "stream_function": aopenai_stream_chat,
@@ -133,6 +195,16 @@ AMAP_PROVIDER_TO_FUNCTION_NAME = {
             "stream_function": aopenai_stream_completion,
         },
     },
+    "openai.azure:responses": {
+        "chat": {
+            "function_name": "openai.AzureOpenAI.responses.create",
+            "stream_function": aopenai_responses_stream_chat,
+        },
+        "completion": {
+            "function_name": "openai.AzureOpenAI.responses.create",
+            "stream_function": aopenai_responses_stream_chat,
+        },
+    },
     "mistral": {
         "chat": {
             "function_name": "mistral.client.chat",
@@ -153,6 +225,26 @@ AMAP_PROVIDER_TO_FUNCTION_NAME = {
             "stream_function": agoogle_stream_completion,
         },
     },
+    "amazon.bedrock": {
+        "chat": {
+            "function_name": "boto3.bedrock-runtime.converse",
+            "stream_function": abedrock_stream_message,
+        },
+        "completion": {
+            "function_name": "boto3.bedrock-runtime.converse",
+            "stream_function": abedrock_stream_message,
+        },
+    },
+    "anthropic.bedrock": {
+        "chat": {
+            "function_name": "anthropic.messages.create",
+            "stream_function": aanthropic_stream_message,
+        },
+        "completion": {
+            "function_name": "anthropic.completions.create",
+            "stream_function": aanthropic_stream_completion,
+        },
+    },
 }
 
 
@@ -163,6 +255,8 @@ AMAP_PROVIDER_TO_FUNCTION = {
     "openai": aopenai_request,
     "openai.azure": aazure_openai_request,
     "vertexai": avertexai_request,
+    "amazon.bedrock": aamazon_bedrock_request,
+    "anthropic.bedrock": aanthropic_bedrock_request,
 }
 
 
@@ -189,6 +283,7 @@ class PromptLayerMixin:
         metadata: Union[Dict[str, str], None],
         provider: Union[str, None] = None,
         model: Union[str, None] = None,
+        model_parameter_overrides: Union[Dict[str, Any], None] = None,
     ) -> Dict[str, Any]:
         params = {}
 
@@ -204,6 +299,8 @@ class PromptLayerMixin:
             params["provider"] = provider
         if model:
             params["model"] = model
+        if model_parameter_overrides:
+            params["model_parameter_overrides"] = model_parameter_overrides
 
         return params
 
@@ -213,7 +310,6 @@ class PromptLayerMixin:
         prompt_blueprint,
         prompt_template,
         prompt_blueprint_model,
-        model_parameter_overrides,
         stream,
         is_async=False,
     ):
@@ -221,6 +317,7 @@ class PromptLayerMixin:
         function_kwargs = deepcopy(prompt_blueprint["llm_kwargs"])
         function_kwargs["stream"] = stream
         provider = prompt_blueprint_model["provider"]
+        api_type = prompt_blueprint_model["api_type"]
 
         if custom_provider := prompt_blueprint.get("custom_provider"):
             provider = custom_provider["client"]
@@ -231,10 +328,7 @@ class PromptLayerMixin:
         elif provider_base_url := prompt_blueprint.get("provider_base_url"):
             client_kwargs["base_url"] = provider_base_url["url"]
 
-        if model_parameter_overrides:
-            function_kwargs.update(model_parameter_overrides)
-
-        if stream and provider in ["openai", "openai.azure"]:
+        if stream and provider in ["openai", "openai.azure"] and api_type == "chat-completions":
             function_kwargs["stream_options"] = {"include_usage": True}
 
         provider_function_name = provider
@@ -244,6 +338,9 @@ class PromptLayerMixin:
             elif "claude" in prompt_blueprint_model["name"]:
                 provider_function_name = "anthropic"
 
+        if provider_function_name in ("openai", "openai.azure"):
+            provider_function_name = f"{provider_function_name}:{api_type}"
+
         if is_async:
             config = AMAP_PROVIDER_TO_FUNCTION_NAME[provider_function_name][prompt_template["type"]]
             request_function = AMAP_PROVIDER_TO_FUNCTION[provider]
