@@ -9,7 +9,7 @@ import sys
 import types
 from copy import deepcopy
 from enum import Enum
-from typing import Any, AsyncGenerator, AsyncIterable, Callable, Dict, Generator, List, Optional, Union
+from typing import Any, Dict, List, Optional, Union
 from uuid import uuid4
 
 import httpx
@@ -59,6 +59,10 @@ def _make_httpx_client():
     return httpx.AsyncClient(timeout=_get_http_timeout())
 
 
+def _make_simple_httpx_client():
+    return httpx.Client(timeout=_get_http_timeout())
+
+
 def _get_workflow_workflow_id_or_name(workflow_id_or_name, workflow_name):
     # This is backward compatibility code
     if (workflow_id_or_name := workflow_name if workflow_id_or_name is None else workflow_id_or_name) is None:
@@ -1189,13 +1193,16 @@ async def apublish_prompt_template(
 
 
 def get_all_prompt_templates(
-    page: int = 1, per_page: int = 30, api_key: str = None
+    page: int = 1, per_page: int = 30, api_key: str = None, label: str = None
 ) -> List[ListPromptTemplateResponse]:
     try:
+        params = {"page": page, "per_page": per_page}
+        if label:
+            params["label"] = label
         response = requests.get(
             f"{URL_API_PROMPTLAYER}/prompt-templates",
             headers={"X-API-KEY": api_key},
-            params={"page": page, "per_page": per_page},
+            params=params,
         )
         if response.status_code != 200:
             raise Exception(
@@ -1208,14 +1215,17 @@ def get_all_prompt_templates(
 
 
 async def aget_all_prompt_templates(
-    page: int = 1, per_page: int = 30, api_key: str = None
+    page: int = 1, per_page: int = 30, api_key: str = None, label: str = None
 ) -> List[ListPromptTemplateResponse]:
     try:
+        params = {"page": page, "per_page": per_page}
+        if label:
+            params["label"] = label
         async with _make_httpx_client() as client:
             response = await client.get(
                 f"{URL_API_PROMPTLAYER}/prompt-templates",
                 headers={"X-API-KEY": api_key},
-                params={"page": page, "per_page": per_page},
+                params=params,
             )
 
         if RAISE_FOR_STATUS:
@@ -1231,364 +1241,6 @@ async def aget_all_prompt_templates(
         raise Exception(f"PromptLayer had the following error while getting all your prompt templates: {str(e)}") from e
 
 
-def openai_stream_chat(results: list):
-    from openai.types.chat import (
-        ChatCompletion,
-        ChatCompletionChunk,
-        ChatCompletionMessage,
-        ChatCompletionMessageToolCall,
-    )
-    from openai.types.chat.chat_completion import Choice
-    from openai.types.chat.chat_completion_message_tool_call import Function
-
-    chat_completion_chunks: List[ChatCompletionChunk] = results
-    response: ChatCompletion = ChatCompletion(
-        id="",
-        object="chat.completion",
-        choices=[
-            Choice(
-                finish_reason="stop",
-                index=0,
-                message=ChatCompletionMessage(role="assistant"),
-            )
-        ],
-        created=0,
-        model="",
-    )
-    last_result = chat_completion_chunks[-1]
-    response.id = last_result.id
-    response.created = last_result.created
-    response.model = last_result.model
-    response.system_fingerprint = last_result.system_fingerprint
-    response.usage = last_result.usage
-    content = ""
-    tool_calls: Union[List[ChatCompletionMessageToolCall], None] = None
-    for result in chat_completion_chunks:
-        choices = result.choices
-        if len(choices) == 0:
-            continue
-        if choices[0].delta.content:
-            content = f"{content}{result.choices[0].delta.content}"
-
-        delta = choices[0].delta
-        if delta.tool_calls:
-            tool_calls = tool_calls or []
-            last_tool_call = None
-            if len(tool_calls) > 0:
-                last_tool_call = tool_calls[-1]
-            tool_call = delta.tool_calls[0]
-            if not tool_call.function:
-                continue
-            if not last_tool_call or tool_call.id:
-                tool_calls.append(
-                    ChatCompletionMessageToolCall(
-                        id=tool_call.id or "",
-                        function=Function(
-                            name=tool_call.function.name or "",
-                            arguments=tool_call.function.arguments or "",
-                        ),
-                        type=tool_call.type or "function",
-                    )
-                )
-                continue
-            last_tool_call.function.name = f"{last_tool_call.function.name}{tool_call.function.name or ''}"
-            last_tool_call.function.arguments = (
-                f"{last_tool_call.function.arguments}{tool_call.function.arguments or ''}"
-            )
-
-    response.choices[0].message.content = content
-    response.choices[0].message.tool_calls = tool_calls
-    return response
-
-
-async def aopenai_stream_chat(generator: AsyncIterable[Any]) -> Any:
-    from openai.types.chat import (
-        ChatCompletion,
-        ChatCompletionChunk,
-        ChatCompletionMessage,
-        ChatCompletionMessageToolCall,
-    )
-    from openai.types.chat.chat_completion import Choice
-    from openai.types.chat.chat_completion_message_tool_call import Function
-
-    chat_completion_chunks: List[ChatCompletionChunk] = []
-    response: ChatCompletion = ChatCompletion(
-        id="",
-        object="chat.completion",
-        choices=[
-            Choice(
-                finish_reason="stop",
-                index=0,
-                message=ChatCompletionMessage(role="assistant"),
-            )
-        ],
-        created=0,
-        model="",
-    )
-    content = ""
-    tool_calls: Union[List[ChatCompletionMessageToolCall], None] = None
-
-    async for result in generator:
-        chat_completion_chunks.append(result)
-        choices = result.choices
-        if len(choices) == 0:
-            continue
-        if choices[0].delta.content:
-            content = f"{content}{choices[0].delta.content}"
-
-        delta = choices[0].delta
-        if delta.tool_calls:
-            tool_calls = tool_calls or []
-            last_tool_call = None
-            if len(tool_calls) > 0:
-                last_tool_call = tool_calls[-1]
-            tool_call = delta.tool_calls[0]
-            if not tool_call.function:
-                continue
-            if not last_tool_call or tool_call.id:
-                tool_calls.append(
-                    ChatCompletionMessageToolCall(
-                        id=tool_call.id or "",
-                        function=Function(
-                            name=tool_call.function.name or "",
-                            arguments=tool_call.function.arguments or "",
-                        ),
-                        type=tool_call.type or "function",
-                    )
-                )
-                continue
-            last_tool_call.function.name = f"{last_tool_call.function.name}{tool_call.function.name or ''}"
-            last_tool_call.function.arguments = (
-                f"{last_tool_call.function.arguments}{tool_call.function.arguments or ''}"
-            )
-
-    # After collecting all chunks, set the response attributes
-    if chat_completion_chunks:
-        last_result = chat_completion_chunks[-1]
-        response.id = last_result.id
-        response.created = last_result.created
-        response.model = last_result.model
-        response.system_fingerprint = getattr(last_result, "system_fingerprint", None)
-        response.usage = last_result.usage
-
-    response.choices[0].message.content = content
-    response.choices[0].message.tool_calls = tool_calls
-    return response
-
-
-def openai_stream_completion(results: list):
-    from openai.types.completion import Completion, CompletionChoice
-
-    completions: List[Completion] = results
-    last_chunk = completions[-1]
-    response = Completion(
-        id=last_chunk.id,
-        created=last_chunk.created,
-        model=last_chunk.model,
-        object="text_completion",
-        choices=[CompletionChoice(finish_reason="stop", index=0, text="")],
-    )
-    text = ""
-    for completion in completions:
-        usage = completion.usage
-        system_fingerprint = completion.system_fingerprint
-        if len(completion.choices) > 0 and completion.choices[0].text:
-            text = f"{text}{completion.choices[0].text}"
-        if usage:
-            response.usage = usage
-        if system_fingerprint:
-            response.system_fingerprint = system_fingerprint
-    response.choices[0].text = text
-    return response
-
-
-async def aopenai_stream_completion(generator: AsyncIterable[Any]) -> Any:
-    from openai.types.completion import Completion, CompletionChoice
-
-    completions: List[Completion] = []
-    text = ""
-    response = Completion(
-        id="",
-        created=0,
-        model="",
-        object="text_completion",
-        choices=[CompletionChoice(finish_reason="stop", index=0, text="")],
-    )
-
-    async for completion in generator:
-        completions.append(completion)
-        usage = completion.usage
-        system_fingerprint = getattr(completion, "system_fingerprint", None)
-        if len(completion.choices) > 0 and completion.choices[0].text:
-            text = f"{text}{completion.choices[0].text}"
-        if usage:
-            response.usage = usage
-        if system_fingerprint:
-            response.system_fingerprint = system_fingerprint
-
-    # After collecting all completions, set the response attributes
-    if completions:
-        last_chunk = completions[-1]
-        response.id = last_chunk.id
-        response.created = last_chunk.created
-        response.model = last_chunk.model
-
-    response.choices[0].text = text
-    return response
-
-
-def anthropic_stream_message(results: list):
-    from anthropic.types import Message, MessageStreamEvent, Usage
-
-    message_stream_events: List[MessageStreamEvent] = results
-    response: Message = Message(
-        id="",
-        model="",
-        content=[],
-        role="assistant",
-        type="message",
-        stop_reason="stop_sequence",
-        stop_sequence=None,
-        usage=Usage(input_tokens=0, output_tokens=0),
-    )
-
-    for event in message_stream_events:
-        if event.type == "message_start":
-            response = event.message
-            break
-
-    content_blocks, usage, stop_reason = build_anthropic_content_blocks(message_stream_events)
-    response.content = content_blocks
-    if usage:
-        response.usage.output_tokens = usage.output_tokens
-    if stop_reason:
-        response.stop_reason = stop_reason
-
-    return response
-
-
-async def aanthropic_stream_message(generator: AsyncIterable[Any]) -> Any:
-    from anthropic.types import Message, MessageStreamEvent, Usage
-
-    message_stream_events: List[MessageStreamEvent] = []
-    response: Message = Message(
-        id="",
-        model="",
-        content=[],
-        role="assistant",
-        type="message",
-        stop_reason="stop_sequence",
-        stop_sequence=None,
-        usage=Usage(input_tokens=0, output_tokens=0),
-    )
-
-    async for event in generator:
-        if event.type == "message_start":
-            response = event.message
-        message_stream_events.append(event)
-
-    content_blocks, usage, stop_reason = build_anthropic_content_blocks(message_stream_events)
-    response.content = content_blocks
-    if usage:
-        response.usage.output_tokens = usage.output_tokens
-    if stop_reason:
-        response.stop_reason = stop_reason
-
-    return response
-
-
-def anthropic_stream_completion(results: list):
-    from anthropic.types import Completion
-
-    completions: List[Completion] = results
-    last_chunk = completions[-1]
-    response = Completion(
-        id=last_chunk.id,
-        completion="",
-        model=last_chunk.model,
-        stop_reason="stop",
-        type="completion",
-    )
-
-    text = ""
-    for completion in completions:
-        text = f"{text}{completion.completion}"
-    response.completion = text
-    return response
-
-
-async def aanthropic_stream_completion(generator: AsyncIterable[Any]) -> Any:
-    from anthropic.types import Completion
-
-    completions: List[Completion] = []
-    text = ""
-    response = Completion(
-        id="",
-        completion="",
-        model="",
-        stop_reason="stop",
-        type="completion",
-    )
-
-    async for completion in generator:
-        completions.append(completion)
-        text = f"{text}{completion.completion}"
-
-    # After collecting all completions, set the response attributes
-    if completions:
-        last_chunk = completions[-1]
-        response.id = last_chunk.id
-        response.model = last_chunk.model
-
-    response.completion = text
-    return response
-
-
-def stream_response(*, generator: Generator, after_stream: Callable, map_results: Callable):
-    data = {
-        "request_id": None,
-        "raw_response": None,
-        "prompt_blueprint": None,
-    }
-    results = []
-    for result in generator:
-        results.append(result)
-        data["raw_response"] = result
-        yield data
-    request_response = map_results(results)
-    response = after_stream(request_response=request_response.model_dump())
-    data["request_id"] = response.get("request_id")
-    data["prompt_blueprint"] = response.get("prompt_blueprint")
-    yield data
-
-
-async def astream_response(
-    generator: AsyncIterable[Any],
-    after_stream: Callable[..., Any],
-    map_results: Callable[[Any], Any],
-) -> AsyncGenerator[Dict[str, Any], None]:
-    data = {
-        "request_id": None,
-        "raw_response": None,
-        "prompt_blueprint": None,
-    }
-    results = []
-    async for result in generator:
-        results.append(result)
-        data["raw_response"] = result
-        yield data
-
-    async def async_generator_from_list(lst):
-        for item in lst:
-            yield item
-
-    request_response = await map_results(async_generator_from_list(results))
-    after_stream_response = await after_stream(request_response=request_response.model_dump())
-    data["request_id"] = after_stream_response.get("request_id")
-    data["prompt_blueprint"] = after_stream_response.get("prompt_blueprint")
-    yield data
-
-
 def openai_chat_request(client, **kwargs):
     return client.chat.completions.create(**kwargs)
 
@@ -1607,8 +1259,13 @@ def openai_request(prompt_blueprint: GetPromptTemplateResponse, client_kwargs: d
     from openai import OpenAI
 
     client = OpenAI(**client_kwargs)
-    request_to_make = MAP_TYPE_TO_OPENAI_FUNCTION[prompt_blueprint["prompt_template"]["type"]]
-    return request_to_make(client, **function_kwargs)
+    api_type = prompt_blueprint["metadata"]["model"]["api_type"]
+
+    if api_type == "chat-completions":
+        request_to_make = MAP_TYPE_TO_OPENAI_FUNCTION[prompt_blueprint["prompt_template"]["type"]]
+        return request_to_make(client, **function_kwargs)
+    else:
+        return client.responses.create(**function_kwargs)
 
 
 async def aopenai_chat_request(client, **kwargs):
@@ -1629,16 +1286,26 @@ async def aopenai_request(prompt_blueprint: GetPromptTemplateResponse, client_kw
     from openai import AsyncOpenAI
 
     client = AsyncOpenAI(**client_kwargs)
-    request_to_make = AMAP_TYPE_TO_OPENAI_FUNCTION[prompt_blueprint["prompt_template"]["type"]]
-    return await request_to_make(client, **function_kwargs)
+    api_type = prompt_blueprint["metadata"]["model"]["api_type"]
+
+    if api_type == "chat-completions":
+        request_to_make = AMAP_TYPE_TO_OPENAI_FUNCTION[prompt_blueprint["prompt_template"]["type"]]
+        return await request_to_make(client, **function_kwargs)
+    else:
+        return await client.responses.create(**function_kwargs)
 
 
 def azure_openai_request(prompt_blueprint: GetPromptTemplateResponse, client_kwargs: dict, function_kwargs: dict):
     from openai import AzureOpenAI
 
     client = AzureOpenAI(azure_endpoint=client_kwargs.pop("base_url", None))
-    request_to_make = MAP_TYPE_TO_OPENAI_FUNCTION[prompt_blueprint["prompt_template"]["type"]]
-    return request_to_make(client, **function_kwargs)
+    api_type = prompt_blueprint["metadata"]["model"]["api_type"]
+
+    if api_type == "chat-completions":
+        request_to_make = MAP_TYPE_TO_OPENAI_FUNCTION[prompt_blueprint["prompt_template"]["type"]]
+        return request_to_make(client, **function_kwargs)
+    else:
+        return client.responses.create(**function_kwargs)
 
 
 async def aazure_openai_request(
@@ -1647,8 +1314,13 @@ async def aazure_openai_request(
     from openai import AsyncAzureOpenAI
 
     client = AsyncAzureOpenAI(azure_endpoint=client_kwargs.pop("base_url", None))
-    request_to_make = AMAP_TYPE_TO_OPENAI_FUNCTION[prompt_blueprint["prompt_template"]["type"]]
-    return await request_to_make(client, **function_kwargs)
+    api_type = prompt_blueprint["metadata"]["model"]["api_type"]
+
+    if api_type == "chat-completions":
+        request_to_make = AMAP_TYPE_TO_OPENAI_FUNCTION[prompt_blueprint["prompt_template"]["type"]]
+        return await request_to_make(client, **function_kwargs)
+    else:
+        return await client.responses.create(**function_kwargs)
 
 
 def anthropic_chat_request(client, **kwargs):
@@ -1754,7 +1426,7 @@ async def autil_log_request(api_key: str, **kwargs) -> Union[RequestLog, None]:
 def mistral_request(prompt_blueprint: GetPromptTemplateResponse, client_kwargs: dict, function_kwargs: dict):
     from mistralai import Mistral
 
-    client = Mistral(api_key=os.environ.get("MISTRAL_API_KEY"))
+    client = Mistral(api_key=os.environ.get("MISTRAL_API_KEY"), client=_make_simple_httpx_client())
     if "stream" in function_kwargs and function_kwargs["stream"]:
         function_kwargs.pop("stream")
         return client.chat.stream(**function_kwargs)
@@ -1770,139 +1442,12 @@ async def amistral_request(
 ):
     from mistralai import Mistral
 
-    client = Mistral(api_key=os.environ.get("MISTRAL_API_KEY"))
+    client = Mistral(api_key=os.environ.get("MISTRAL_API_KEY"), async_client=_make_httpx_client())
     if "stream" in function_kwargs and function_kwargs["stream"]:
         return await client.chat.stream_async(**function_kwargs)
     return await client.chat.complete_async(**function_kwargs)
 
 
-def mistral_stream_chat(results: list):
-    from openai.types.chat import ChatCompletion, ChatCompletionMessage, ChatCompletionMessageToolCall
-    from openai.types.chat.chat_completion import Choice
-    from openai.types.chat.chat_completion_message_tool_call import Function
-
-    last_result = results[-1]
-    response = ChatCompletion(
-        id=last_result.data.id,
-        object="chat.completion",
-        choices=[
-            Choice(
-                finish_reason=last_result.data.choices[0].finish_reason or "stop",
-                index=0,
-                message=ChatCompletionMessage(role="assistant"),
-            )
-        ],
-        created=last_result.data.created,
-        model=last_result.data.model,
-    )
-
-    content = ""
-    tool_calls = None
-
-    for result in results:
-        choices = result.data.choices
-        if len(choices) == 0:
-            continue
-
-        delta = choices[0].delta
-        if delta.content is not None:
-            content = f"{content}{delta.content}"
-
-        if delta.tool_calls:
-            tool_calls = tool_calls or []
-            for tool_call in delta.tool_calls:
-                if len(tool_calls) == 0 or tool_call.id:
-                    tool_calls.append(
-                        ChatCompletionMessageToolCall(
-                            id=tool_call.id or "",
-                            function=Function(
-                                name=tool_call.function.name,
-                                arguments=tool_call.function.arguments,
-                            ),
-                            type="function",
-                        )
-                    )
-                else:
-                    last_tool_call = tool_calls[-1]
-                    if tool_call.function.name:
-                        last_tool_call.function.name = f"{last_tool_call.function.name}{tool_call.function.name}"
-                    if tool_call.function.arguments:
-                        last_tool_call.function.arguments = (
-                            f"{last_tool_call.function.arguments}{tool_call.function.arguments}"
-                        )
-
-    response.choices[0].message.content = content
-    response.choices[0].message.tool_calls = tool_calls
-    response.usage = last_result.data.usage
-    return response
-
-
-async def amistral_stream_chat(generator: AsyncIterable[Any]) -> Any:
-    from openai.types.chat import ChatCompletion, ChatCompletionMessage, ChatCompletionMessageToolCall
-    from openai.types.chat.chat_completion import Choice
-    from openai.types.chat.chat_completion_message_tool_call import Function
-
-    completion_chunks = []
-    response = ChatCompletion(
-        id="",
-        object="chat.completion",
-        choices=[
-            Choice(
-                finish_reason="stop",
-                index=0,
-                message=ChatCompletionMessage(role="assistant"),
-            )
-        ],
-        created=0,
-        model="",
-    )
-    content = ""
-    tool_calls = None
-
-    async for result in generator:
-        completion_chunks.append(result)
-        choices = result.data.choices
-        if len(choices) == 0:
-            continue
-        delta = choices[0].delta
-        if delta.content is not None:
-            content = f"{content}{delta.content}"
-
-        if delta.tool_calls:
-            tool_calls = tool_calls or []
-            for tool_call in delta.tool_calls:
-                if len(tool_calls) == 0 or tool_call.id:
-                    tool_calls.append(
-                        ChatCompletionMessageToolCall(
-                            id=tool_call.id or "",
-                            function=Function(
-                                name=tool_call.function.name,
-                                arguments=tool_call.function.arguments,
-                            ),
-                            type="function",
-                        )
-                    )
-                else:
-                    last_tool_call = tool_calls[-1]
-                    if tool_call.function.name:
-                        last_tool_call.function.name = f"{last_tool_call.function.name}{tool_call.function.name}"
-                    if tool_call.function.arguments:
-                        last_tool_call.function.arguments = (
-                            f"{last_tool_call.function.arguments}{tool_call.function.arguments}"
-                        )
-
-    if completion_chunks:
-        last_result = completion_chunks[-1]
-        response.id = last_result.data.id
-        response.created = last_result.data.created
-        response.model = last_result.data.model
-        response.usage = last_result.data.usage
-
-    response.choices[0].message.content = content
-    response.choices[0].message.tool_calls = tool_calls
-    return response
-
-
 def google_chat_request(client, **kwargs):
     from google.genai.chats import Content
 
@@ -1911,7 +1456,7 @@ def google_chat_request(client, **kwargs):
     history = [Content(**item) for item in kwargs.get("history", [])]
     generation_config = kwargs.get("generation_config", {})
     chat = client.chats.create(model=model, history=history, config=generation_config)
-    last_message = history[-1].parts[0] if history else ""
+    last_message = history[-1].parts if history else ""
     if stream:
         return chat.send_message_stream(message=last_message)
     return chat.send_message(message=last_message)
@@ -1927,65 +1472,6 @@ def google_completions_request(client, **kwargs):
     return client.models.generate_content(model=model, contents=contents, config=config)
 
 
-def _build_google_response_from_parts(thought_content: str, regular_content: str, function_calls: list, last_result):
-    """Helper function to build Google response with thought, regular, and function call parts."""
-    from google.genai.chats import Part
-
-    response = last_result.model_copy()
-    final_parts = []
-
-    if thought_content:
-        thought_part = Part(text=thought_content, thought=True)
-        final_parts.append(thought_part)
-
-    if regular_content:
-        text_part = Part(text=regular_content, thought=None)
-        final_parts.append(text_part)
-
-    for function_call in function_calls:
-        function_part = Part(function_call=function_call, thought=None)
-        final_parts.append(function_part)
-
-    if final_parts:
-        response.candidates[0].content.parts = final_parts
-
-    return response
-
-
-def map_google_stream_response(results: list):
-    from google.genai.chats import GenerateContentResponse
-
-    response = GenerateContentResponse()
-    if not results:
-        return response
-    results: List[GenerateContentResponse] = results
-
-    thought_content = ""
-    regular_content = ""
-    function_calls = []
-
-    for result in results:
-        if result.candidates and result.candidates[0].content.parts:
-            for part in result.candidates[0].content.parts:
-                if hasattr(part, "text") and part.text:
-                    if hasattr(part, "thought") and part.thought:
-                        thought_content = f"{thought_content}{part.text}"
-                    else:
-                        regular_content = f"{regular_content}{part.text}"
-                elif hasattr(part, "function_call") and part.function_call:
-                    function_calls.append(part.function_call)
-
-    return _build_google_response_from_parts(thought_content, regular_content, function_calls, results[-1])
-
-
-def google_stream_chat(results: list):
-    return map_google_stream_response(results)
-
-
-def google_stream_completion(results: list):
-    return map_google_stream_response(results)
-
-
 MAP_TYPE_TO_GOOGLE_FUNCTION = {
     "chat": google_chat_request,
     "completion": google_completions_request,
@@ -1995,7 +1481,14 @@ MAP_TYPE_TO_GOOGLE_FUNCTION = {
 def google_request(prompt_blueprint: GetPromptTemplateResponse, client_kwargs: dict, function_kwargs: dict):
     from google import genai
 
-    client = genai.Client()
+    if os.environ.get("GOOGLE_GENAI_USE_VERTEXAI") == "true":
+        client = genai.Client(
+            vertexai=True,
+            project=os.environ.get("GOOGLE_CLOUD_PROJECT"),
+            location=os.environ.get("GOOGLE_CLOUD_LOCATION"),
+        )
+    else:
+        client = genai.Client(api_key=os.environ.get("GEMINI_API_KEY") or os.environ.get("GOOGLE_API_KEY"))
     request_to_make = MAP_TYPE_TO_GOOGLE_FUNCTION[prompt_blueprint["prompt_template"]["type"]]
     return request_to_make(client, **function_kwargs)
 
@@ -2033,47 +1526,18 @@ AMAP_TYPE_TO_GOOGLE_FUNCTION = {
 async def agoogle_request(prompt_blueprint: GetPromptTemplateResponse, client_kwargs: dict, function_kwargs: dict):
     from google import genai
 
-    client = genai.Client()
+    if os.environ.get("GOOGLE_GENAI_USE_VERTEXAI") == "true":
+        client = genai.Client(
+            vertexai=True,
+            project=os.environ.get("GOOGLE_CLOUD_PROJECT"),
+            location=os.environ.get("GOOGLE_CLOUD_LOCATION"),
+        )
+    else:
+        client = genai.Client(api_key=os.environ.get("GEMINI_API_KEY") or os.environ.get("GOOGLE_API_KEY"))
     request_to_make = AMAP_TYPE_TO_GOOGLE_FUNCTION[prompt_blueprint["prompt_template"]["type"]]
     return await request_to_make(client, **function_kwargs)
 
 
-async def amap_google_stream_response(generator: AsyncIterable[Any]):
-    from google.genai.chats import GenerateContentResponse
-
-    response = GenerateContentResponse()
-
-    thought_content = ""
-    regular_content = ""
-    function_calls = []
-    last_result = None
-
-    async for result in generator:
-        last_result = result
-        if result.candidates and result.candidates[0].content.parts:
-            for part in result.candidates[0].content.parts:
-                if hasattr(part, "text") and part.text:
-                    if hasattr(part, "thought") and part.thought:
-                        thought_content = f"{thought_content}{part.text}"
-                    else:
-                        regular_content = f"{regular_content}{part.text}"
-                elif hasattr(part, "function_call") and part.function_call:
-                    function_calls.append(part.function_call)
-
-    if not last_result:
-        return response
-
-    return _build_google_response_from_parts(thought_content, regular_content, function_calls, last_result)
-
-
-async def agoogle_stream_chat(generator: AsyncIterable[Any]):
-    return await amap_google_stream_response(generator)
-
-
-async def agoogle_stream_completion(generator: AsyncIterable[Any]):
-    return await amap_google_stream_response(generator)
-
-
 def vertexai_request(prompt_blueprint: GetPromptTemplateResponse, client_kwargs: dict, function_kwargs: dict):
     if "gemini" in prompt_blueprint["metadata"]["model"]["name"]:
         return google_request(
@@ -2118,3 +1582,90 @@ async def avertexai_request(prompt_blueprint: GetPromptTemplateResponse, client_
     raise NotImplementedError(
         f"Vertex AI request for model {prompt_blueprint['metadata']['model']['name']} is not implemented yet."
     )
+
+
+def amazon_bedrock_request(prompt_blueprint: GetPromptTemplateResponse, client_kwargs: dict, function_kwargs: dict):
+    import boto3
+
+    bedrock_client = boto3.client(
+        "bedrock-runtime",
+        aws_access_key_id=function_kwargs.pop("aws_access_key", None),
+        aws_secret_access_key=function_kwargs.pop("aws_secret_key", None),
+        region_name=function_kwargs.pop("aws_region", "us-east-1"),
+    )
+
+    stream = function_kwargs.pop("stream", False)
+
+    if stream:
+        return bedrock_client.converse_stream(**function_kwargs)
+    else:
+        return bedrock_client.converse(**function_kwargs)
+
+
+async def aamazon_bedrock_request(
+    prompt_blueprint: GetPromptTemplateResponse, client_kwargs: dict, function_kwargs: dict
+):
+    import aioboto3
+
+    aws_access_key = function_kwargs.pop("aws_access_key", None)
+    aws_secret_key = function_kwargs.pop("aws_secret_key", None)
+    aws_region = function_kwargs.pop("aws_region", "us-east-1")
+
+    session_kwargs = {}
+    if aws_access_key:
+        session_kwargs["aws_access_key_id"] = aws_access_key
+    if aws_secret_key:
+        session_kwargs["aws_secret_access_key"] = aws_secret_key
+    if aws_region:
+        session_kwargs["region_name"] = aws_region
+
+    stream = function_kwargs.pop("stream", False)
+    session = aioboto3.Session()
+
+    async with session.client("bedrock-runtime", **session_kwargs) as client:
+        if stream:
+            return await client.converse_stream(**function_kwargs)
+        else:
+            return await client.converse(**function_kwargs)
+
+
+def anthropic_bedrock_request(prompt_blueprint: GetPromptTemplateResponse, client_kwargs: dict, function_kwargs: dict):
+    from anthropic import AnthropicBedrock
+
+    client = AnthropicBedrock(
+        aws_access_key=function_kwargs.pop("aws_access_key", None),
+        aws_secret_key=function_kwargs.pop("aws_secret_key", None),
+        aws_region=function_kwargs.pop("aws_region", None),
+        aws_session_token=function_kwargs.pop("aws_session_token", None),
+        base_url=function_kwargs.pop("base_url", None),
+        **client_kwargs,
+    )
+    if prompt_blueprint["prompt_template"]["type"] == "chat":
+        return anthropic_chat_request(client=client, **function_kwargs)
+    elif prompt_blueprint["prompt_template"]["type"] == "completion":
+        return anthropic_completions_request(client=client, **function_kwargs)
+    raise NotImplementedError(
+        f"Unsupported prompt template type {prompt_blueprint['prompt_template']['type']}' for Anthropic Bedrock"
+    )
+
+
+async def aanthropic_bedrock_request(
+    prompt_blueprint: GetPromptTemplateResponse, client_kwargs: dict, function_kwargs: dict
+):
+    from anthropic import AsyncAnthropicBedrock
+
+    client = AsyncAnthropicBedrock(
+        aws_access_key=function_kwargs.pop("aws_access_key", None),
+        aws_secret_key=function_kwargs.pop("aws_secret_key", None),
+        aws_region=function_kwargs.pop("aws_region", None),
+        aws_session_token=function_kwargs.pop("aws_session_token", None),
+        base_url=function_kwargs.pop("base_url", None),
+        **client_kwargs,
+    )
+    if prompt_blueprint["prompt_template"]["type"] == "chat":
+        return await aanthropic_chat_request(client=client, **function_kwargs)
+    elif prompt_blueprint["prompt_template"]["type"] == "completion":
+        return await aanthropic_completions_request(client=client, **function_kwargs)
+    raise NotImplementedError(
+        f"Unsupported prompt template type {prompt_blueprint['prompt_template']['type']}' for Anthropic Bedrock"
+    )
